from abc import ABCMeta, abstractmethod
from enum import Enum, unique


@unique
class DatumType(Enum):
    FUND = 'FUND'


class BusinessException(Exception):
    def __init__(self, msg):
        self.__msg = msg

    def __str__(self):
        return self.__msg


class Datum(metaclass=ABCMeta):
    '''
    基础资料服务，基金资料数据，各种指数，指标资料数据
    '''
    @abstractmethod
    def get_fund_datums(self, crncy=None, risk=None, fund_ids=None):
        '''
        获取基金资料数据
        :param crncy: 货币类型
        :param risk: 风险等级，e.g: 1-5
        :param fund_ids: 基金ID列表
        :return:基金资料信息
        '''
        pass


class Navs(metaclass=ABCMeta):
    '''
    基础数据相关服务，基金净值，各种指标 高开低收
    '''
    @abstractmethod
    def get_fund_navs(self, fund_ids=None, min_date=None, max_date=None):
        '''
        获取基金净值信息
        :param fund_ids: 基金id，可以多个id，使用tuple包裹
        :param min_date: 起始时间
        :param max_date: 截止时间
        :return: 基金净值信息
        '''
        pass


class FundOptimize(metaclass=ABCMeta):
    '''
    基金优选相关服务
    '''
    @abstractmethod
    def find_optimize(self, fund_ids, day):
        '''
        从多个基金id中，选出指定日期最优的基金id
        :param fund_ids: 待选基金id列表
        :param day: 指定日期
        :return: 最优的基金id
        '''
        pass

    @abstractmethod
    def get_optimize_pool(self, day):
        '''
        根据基金优选获取指定日期的基金池
        :param day: 指定日期
        :return: 基金id列表
        '''
        pass
